#include "DataStreamingListener.h"
#include <fstream>
#include <ctime>
#include <sstream>

using namespace std;

DataStreamingListener::DataStreamingListener(bool writeToFile, std::string path)
{
    m_streamASCII = 1;
    m_writeToFile = writeToFile;
    if (writeToFile)
    {
        m_outputFile.open(path.c_str());
    }
}

DataStreamingListener::~DataStreamingListener()
{
    if (m_writeToFile)
        m_outputFile.close();
}

void DataStreamingListener::onStreamingMessage(const char* msg, const unsigned int length, const char* msgKey)
{
    if (m_writeToFile)
    {
        m_outputFile << msg << endl;
    }
    else
    {
        lockStdOut();
        cout << msg << endl;
        unlockStdOut();
    }
}

void DataStreamingListener::onStreamingError(BPNGError err)
{
    time_t now = time(nullptr);
    cerr << "BPNGError at " << asctime(gmtime(&now)) << " | " << err.code << ": " << err.msg << endl;
    if (m_writeToFile)
        m_outputFile << "BPNGError at " << asctime(gmtime(&now)) << " | " << err.code << ": " << err.msg << endl;
}

void DataStreamingListener::onDataDiscarded(int numBytes, IChannelList* channels)
{
    cout << "Data discarded - bytes: " << numBytes << endl;
    if (m_writeToFile)
        m_outputFile << "Data discarded - bytes: " << numBytes << endl;
}

BOOL DataStreamingListener::isASCIIRequested()
{
    return m_streamASCII;
}

void DataStreamingListener::onOnlineStreamingDisabled(uint32_t mbnr)
{
    cout << "Online Streaming disabled for mbnr " << mbnr;
    if (m_writeToFile)
        m_outputFile << "Online streaming disabled for mbnr " << mbnr << endl;
}
